@@ -110,7 +110,7 @@ def encrypt(
         verify_dtr = not offline and dtr_id is not None
     if offline and verify_dtr:
         raise UserError(
-            "--verify_dtr is not possible when offline is " "enabled in the config"
+            "--verify_dtr is not possible when offline is enabled in the config"
         )
     if config.sign_encrypted_data and not dry_run:
         kwargs["passphrase"] = get_passphrase_from_cmd_or_prompt(
@@ -247,18 +247,20 @@ class Cli(Subcommands):
                 ),
                 "verify_dtr": verify_dtr_override,
                 "purpose": dict(
-                    help="Purpose of the DTR (PRODUCTION, TEST)."
+                    help="Purpose of the DTR (PRODUCTION, TEST). "
                     "Mandatory only if `transfer_id` is specified."
                 ),
-                "output_name": dict(
-                    help="output encrypted file name. If no path is specified, "
-                    "the output tarball is saved in the current working "
-                    "directory. If this argument is "
-                    "missing the output name is set to a string based on "
-                    "the current date and time the function.",
+                "output": dict(
+                    help="output path (file or directory) to the "
+                    "encrypted package. If no path is specified, "
+                    "the output package is saved in the current working "
+                    "directory. If this argument is missing or path is a "
+                    "directory the file name is generated based on the "
+                    "current date and an optional suffix.",
                     default=None,
                     alias="-o",
                 ),
+                "output_suffix": dict(help="output file name suffix (optional)"),
                 "dry_run": dry_run_override,
                 "compression_level": dict(
                     help="compression level for inner tarball in the range "
