@@ -1,22 +1,19 @@
 import enum
 import io
-import os
 import re
 import urllib
 import warnings
-from pathlib import Path
 from typing import List, Optional, Tuple, Union, Type, IO, Callable, cast
 from functools import wraps, partial
 
 import gpg_lite as gpg
+from gpg_lite import GPGStore
 from libbiomedit import crypt
 from .error import UserError
 from .request import urlopen
 from .secret import Secret
-
-GPGStore = gpg.GPGStore
-extract_pub_key_ids = gpg.extract_key_id
-get_default_gpg_config_dir = gpg.get_default_gpg_config_dir
+from ..utils.config import Config
+from .. import APP_NAME_SHORT
 
 ExceptionType = Union[Type[BaseException], Tuple[Type[BaseException], ...]]
 
@@ -54,9 +51,9 @@ def to_user_error_gen(error_types: ExceptionType):
 
 
 validate_pub_key = to_user_error(RuntimeError)(crypt.validate_pub_key)
-validated_keys_by_ids = to_user_error_gen((RuntimeError, gpg.cmd.GPGError))(
-    partial(crypt.validated_keys_by_ids, url_opener=urlopen)
-)
+retrieve_refresh_and_validate_keys = to_user_error_gen(
+    (RuntimeError, gpg.cmd.GPGError)
+)(partial(crypt.retrieve_refresh_and_validate_keys, url_opener=urlopen))
 refresh_keys = to_user_error((RuntimeError, gpg.cmd.GPGError))(
     partial(crypt.refresh_keys, url_opener=urlopen)
 )
@@ -67,9 +64,12 @@ verify_metadata_signature = to_user_error(RuntimeError)(
 
 
 @to_user_error(gpg.cmd.GPGError)
-def search_keyserver(name: str, keyserver: str) -> List[gpg.KeyInfo]:
-    """Search keyserver for keys maching the query"""
-    return list(gpg.search_keys(name, keyserver, url_opener=urlopen))
+def search_keyserver(search_term: str, keyserver: str) -> List[gpg.KeyInfo]:
+    """Search keyserver for keys matching the query"""
+    try:
+        return list(gpg.search_keyserver(search_term, keyserver, url_opener=urlopen))
+    except urllib.error.URLError:
+        raise KeyserverError(action="search", keyserver=keyserver) from None
 
 
 @to_user_error(gpg.cmd.GPGError)
@@ -79,9 +79,9 @@ def import_keys(key_data: str, gpg_store: GPGStore):
 
 
 @to_user_error(gpg.cmd.GPGError)
-def delete_keys(fingerprints: List[str], gpg_store: GPGStore):
+def delete_pub_keys(fingerprints: List[str], gpg_store: GPGStore):
     """Delete public key"""
-    return gpg_store.delete_keys(*fingerprints)
+    return gpg_store.delete_pub_keys(*fingerprints)
 
 
 @to_user_error(gpg.cmd.GPGError)
@@ -133,7 +133,9 @@ def detach_sign_file(
 def check_password(user_fingerprint: str, passphrase: str, gpg_store: GPGStore):
     """Check if key password is correct"""
     try:
-        with gpg_store.detach_sig(b"test", passphrase, user_fingerprint):
+        with gpg_store.detach_sig(
+            src=b"test", passphrase=passphrase, signee=user_fingerprint
+        ):
             pass
     except gpg.cmd.GPGError:
         raise UserError("GPG password is incorrect") from None
@@ -144,79 +146,6 @@ class KeyType(enum.Enum):
     secret = enum.auto()
 
 
-def open_gpg_dir(gpg_dir: str) -> GPGStore:
-    """Open the database inside a GnuPG directory and return it as a gpg
-    object.
-
-    :param gpg_dir: path of the GnuPG directory to open.
-    :return: a gnupg GPGStore
-    :raises UserError:
-    """
-    if not Path(gpg_dir).is_dir():
-        os.makedirs(gpg_dir, mode=0o700)
-
-    try:
-        return GPGStore(config_dir=gpg_dir)
-    except ValueError:
-        raise UserError(f"unable to open GnuPG directory [{gpg_dir}].") from None
-
-
-def retrieve_keys(
-    gpg_store: GPGStore,
-    sender: str,
-    recipients: List[str],
-    validation_keyid: Optional[str] = None,
-    keyserver_url: Optional[str] = None,
-    with_refresh_keys: bool = True,
-) -> Tuple[gpg.Key, List[gpg.Key]]:
-    """Retrieve the following keys from the user's local GnuPG keyring:
-     - public key of sender.
-     - public key of recipient(s).
-    Exactly one key must be found for each sender and recipient, otherwise an
-    error is raised. A number of checks are also performed on the keys. The
-    keys are then returned as a tuple with one sender key, and one or more
-    recipient keys: (sender key, [recipient keys]).
-
-    :param gpg_store: local GnuPG database as gpg-lite object.
-    :param sender: fingerprint or email of data sender.
-    :param recipients: fingerprint or email of data recipient(s). There can
-        be one or more recipients.
-    :param validation_keyid: fingerprint of validation key. Used to check that
-        the retrieved key is signed by the validation key.
-    :param keyserver_url: url of keyserver. Used for checking key origin.
-    :param with_refresh_keys: if True, the found public keys are re-downloaded
-        from the keyserver to make sure they are up-to-date.
-    :return: tuple with one sender key, and one or more recipient keys.
-    :raises UnpackError:
-    """
-    # Search public keys for sender and recipient(s).
-    pub_keys = [
-        search_pub_key(search_term=x, gpg_store=gpg_store, sigs=True)
-        for x in [sender] + recipients
-    ]
-
-    # If asked to do so, refresh keys by re-downloading them from keyserver.
-    if with_refresh_keys:
-        pub_keys = refresh_keys(
-            keys=pub_keys, gpg_store=gpg_store, sigs=True, keyserver_url=keyserver_url
-        )
-    # Verify keys are signed and non-revoked.
-    for key in pub_keys:
-        validate_pub_key(key, validation_keyid, keyserver_url)
-
-    # Split keys into sender and recipient(s) keys.
-    sender_key, *recipient_keys = pub_keys
-
-    # Verify a private key matching the user's public key exists. Note that
-    # the key itself is not really needed because it shares the fingerprint
-    # with the public key, but it has to be there.
-    search_priv_key(search_term=sender_key.fingerprint, gpg_store=gpg_store)
-
-    # Return the sender's public key, and one or more recipient public keys
-    # as two separate objects.
-    return (sender_key, recipient_keys)
-
-
 def search_pub_key(search_term: str, gpg_store: GPGStore, sigs: bool = True) -> gpg.Key:
     """Search for a single public key in the local keyring based on the input
     search term. If no or more than one key is matching the search term, an
@@ -226,10 +155,10 @@ def search_pub_key(search_term: str, gpg_store: GPGStore, sigs: bool = True) ->
     :param search_term: search term for the key, e.g. fingerprint or key owner
         email address.
     :param sigs: if True, return key with signatures.
-    :return: GnuPG key matching the search term.
+    :return: PGP key matching the search term.
     :raises UnpackError:
     """
-    keys = gpg_store.list_pub_keys(keys=(search_term,), sigs=sigs)
+    keys = gpg_store.list_pub_keys(search_terms=(search_term,), sigs=sigs)
     try:
         (key,) = keys
     except ValueError:
@@ -243,7 +172,7 @@ def search_priv_key(search_term: str, gpg_store: GPGStore) -> gpg.Key:
     """Searches the user's local keyring for a secret key matching the search
     term. Raises an error if no or more than one key are found.
     """
-    keys = gpg_store.list_sec_keys(keys=(search_term,))
+    keys = gpg_store.list_sec_keys(search_terms=(search_term,))
     try:
         (key,) = keys
     except ValueError:
@@ -266,16 +195,6 @@ class UnpackError(UserError):
         super().__init__(msg_start + f" matching: {search_value}")
 
 
-def assert_key_is_not_revoked(key: gpg.Key):
-    """Check if a GnuPG key is revoked
-
-    :param key: Key to check
-    :raises UserError:
-    """
-    if key.validity is gpg.Validity.revoked:
-        raise UserError(f"{key.uids[0]} key has been revoked")
-
-
 def encrypt_and_sign(
     source: Callable,
     output: Callable,
@@ -285,12 +204,11 @@ def encrypt_and_sign(
     passphrase: Optional[str] = None,
     always_trust: bool = True,
 ):
-    """Encrypt input data with a GnuPG public key and sign it with a GnuPG
-    private key.
+    """Encrypt input data with a PGP public key and sign it with a private key.
 
     In this function, the compression level of the "encrypt()" method is set
     to 0 as we are only encrypting data that has already been compressed, or
-    has explicitely been requeseted by the user not to be compressed.
+    has explicitely been requested by the user not to be compressed.
 
     There is no check of the validity of the sender and recipient keys, as it
     is assumed that this is already done earlier.
@@ -324,20 +242,23 @@ def encrypt_and_sign(
         )
     except gpg.cmd.GPGError:
         raise UserError(
-            "Encryption failed. " "Maybe you entered a wrong passphrase."
+            "Encryption failed. Maybe you entered a wrong passphrase."
         ) from None
 
 
 def get_recipient_email(key: gpg.Key) -> str:
+    """Retrieve the email address associated with a PGP key, and generate a
+    an error if the email address is missing or could not be retrieved.
+    """
     try:
         email = key.uids[0].email
     except (IndexError, UnpackError):
         raise UserError(
-            f"Could not determine email address " f"for GPG key {key.key_id}."
+            f"Could not determine email address for PGP key {key.key_id}."
         ) from None
     if not email:
         raise UserError(
-            f"GPG key [{key.key_id}] does not contain a valid " f"email address."
+            f"PGP key [{key.key_id}] does not contain a valid email address."
         )
     return email
 
@@ -392,14 +313,14 @@ def create_key(
     key_type: str = "RSA",
     key_length: int = 4096,
 ) -> gpg.Key:
-    """Create a new public/private key"""
+    """Create a new PGP public/private key"""
     min_pwd_len = 10
     if len(full_name) < 5:
         raise UserError("Full name must be at least 5 characters long.")
     if not re.search(r"[^@]+@[^@]+\.[^@]+", email):
         raise UserError("Invalid email address.")
     if pwd != pwd_repeat:
-        raise UserError("Password do not match.")
+        raise UserError("Password and repeated password do not match.")
     if len(pwd) < min_pwd_len:
         raise UserError("Password is too short (min length: " f"{min_pwd_len})")
     fingerprint = gpg_store.gen_key(
@@ -409,7 +330,7 @@ def create_key(
         email=email,
         passphrase=pwd,
     )
-    pkey = gpg_store.list_sec_keys((fingerprint,))
+    pkey = gpg_store.list_sec_keys(search_terms=(fingerprint,))
     if not pkey:
         raise UserError(f"No private keys found for: {fingerprint}")
     if len(pkey) > 1:
@@ -417,19 +338,50 @@ def create_key(
     return pkey[0]
 
 
+def load_authority_key(sett_config: Config) -> Optional[gpg.Key]:
+    """Load (download if needed) and refresh the key authority's PGP key
+
+    The function returns None if any of the following:
+     * No key authority fingerprint is given in the config.
+     * Offline mode is on.
+     * The key matching the fingerprint could not be retrieved.
+    """
+    if sett_config.key_authority_fingerprint and not sett_config.offline:
+        try:
+            return crypt.retrieve_and_refresh_certification_authority_key(
+                key_fingerprint=sett_config.key_authority_fingerprint,
+                gpg_store=sett_config.gpg_store,
+                keyserver_url=sett_config.keyserver_url,
+                validate_key_origin=False,
+                url_opener=urlopen,
+            )
+        except RuntimeError as e:
+            warnings.warn(
+                "The key validation authority's PGP key "
+                f"[{sett_config.key_authority_fingerprint}] could not be "
+                "found in your local keyring, nor could it be downloaded.\n"
+                "Please verify that the key fingerprint and the keyserver URL "
+                "are correct and that you are not in offline mode.\n"
+                f"Reason for failure: {e}"
+            )
+
+    return None
+
+
 class KeyserverError(UserError):
     """Error class that displays an error message when the keyserver cannot
     be reached.
+
     :param action: either "download" or "upload".
     :param keyserver: URL of keyserver.
     """
 
     def __init__(self, action: str, keyserver: str):
         super().__init__(
-            f"Key {action} failed as sett could not connect to the keyserver.\n"
-            f"A possible cause for this is that the keyserver is temporarily "
-            f"unavailable.\n"
+            f"Key {action} failed as {APP_NAME_SHORT} could not connect to the "
+            "keyserver.\n"
+            "A possible cause for this is that the keyserver is temporarily "
+            "unavailable.\n"
             f"Please try to connect to [{keyserver}] using your web browser to "
-            f"confirm whether the keyserver is available or not.\n"
-            f"If the issue persists, please contact the DCC."
+            "confirm whether the keyserver is available or not.\n"
         )
