@@ -1,6 +1,7 @@
 import os
 import shutil
 import warnings
+from contextlib import contextmanager
 from itertools import count
 from pathlib import Path
 from typing import List, Iterator, Union
@@ -54,7 +55,7 @@ def delete_files(*files: str) -> None:
         if os.path.exists(f):
             try:
                 os.unlink(f)
-            except BaseException:
+            except OSError:
                 failed.append(f)
 
     if failed:
@@ -99,3 +100,19 @@ class DeleteDirOnError:
                 shutil.rmtree(self.directory)
             else:
                 logger.debug("Directory doesn't exist: %s", self.directory)
+
+
+@contextmanager
+def delete_file_on_error(path: Union[str, Path]):
+    """Context manager that deletes a file if any exception is raised
+    while in the context manager block.
+    """
+    try:
+        yield
+    except Exception as e:
+        if os.path.exists(path):
+            logger.debug("Deleting file: %s", path)
+            os.remove(path)
+        else:
+            logger.debug("File doesn't exist: %s", path)
+        raise e
