@@ -4,7 +4,7 @@ from typing import Callable, Optional, Tuple
 
 from .model import AppData
 from .parallel import Worker
-from .pyside import QtCore, QtWidgets, QAction, QtGui
+from .pyside import QtCore, QtWidgets, QAction, QtGui, open_window
 from ..utils.progress import ProgressInterface
 
 EXTRA_HEIGHT = 1
@@ -114,7 +114,7 @@ class GuiProgress(QtCore.QObject, ProgressInterface):
 class ConsoleWidget(QtWidgets.QGroupBox):
     def __init__(self, title, parent):
         super().__init__(title, parent)
-        self.textbox = QtWidgets.QPlainTextEdit()
+        self.textbox = QtWidgets.QTextEdit()
         self.textbox.setReadOnly(True)
         btn_clear_console = QtWidgets.QPushButton(
             QtGui.QIcon(":icon/feather/slash.png"), ""
@@ -129,10 +129,10 @@ class ConsoleWidget(QtWidgets.QGroupBox):
         self.textbox.clear()
 
     def append(self, text: str) -> None:
-        self.textbox.appendPlainText(text)
+        self.textbox.append(text)
 
     def write(self, text: str) -> None:
-        self.textbox.appendPlainText(text)
+        self.textbox.append(text)
 
 
 class PathInput:
@@ -238,7 +238,10 @@ def run_dialog(parent, msg: str, password: bool = True):
     dialog_pwd.setWindowTitle("SETT")
     if password:
         dialog_pwd.setTextEchoMode(QtWidgets.QLineEdit.EchoMode.Password)
-    if dialog_pwd.exec_() != QtWidgets.QDialog.Accepted or not dialog_pwd.textValue():
+    if (
+        open_window(dialog_pwd) != QtWidgets.QDialog.Accepted
+        or not dialog_pwd.textValue()
+    ):
         return None
     return dialog_pwd.textValue()
 
@@ -249,8 +252,9 @@ def warning_callback(
     msg_warn = warning_dialog(title, parent=parent)
 
     def _show_warning(msg: str):
-        msg_warn.setText(format(msg))
-        msg_warn.exec_()
+        # add an extra line break so that text is easier to read in GUI.
+        msg_warn.setText(format(msg).replace("\n", "\n\n"))
+        open_window(msg_warn)
 
     return _show_warning
 
