@@ -1,4 +1,3 @@
-from datetime import datetime
 from functools import partial
 
 from .component import (
@@ -18,6 +17,7 @@ from .parallel import Worker
 from .pyside import QtCore, QtGui, QtWidgets
 from .settings_tab import LABEL_COMPRESSION_LEVEL, STATUS_TIP_COMPRESSION_LEVEL
 from ..core.metadata import Purpose
+from ..utils.validation import PACKAGE_SUFFIX
 from ..workflows import encrypt
 
 
@@ -159,11 +159,12 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         group_box = QtWidgets.QGroupBox("Output")
         # Create fields
         output_suffix = LineEdit()
+        output_suffix.setText(self.app_data.encrypt_package_name_suffix)
         output_suffix.setStatusTip(
-            "(optional) File name suffix in the " "format datetime_suffix.tar"
+            "(optional) File name suffix in the format (project_)datetime(_suffix).tar"
         )
         output_suffix.setValidator(
-            QtGui.QRegularExpressionValidator(QtCore.QRegularExpression(r"[\w-]*"))
+            QtGui.QRegularExpressionValidator(QtCore.QRegularExpression(PACKAGE_SUFFIX))
         )
         output_location = PathInput(path=self.app_data.encrypt_output_location)
         output_location.status_tip = "Destination folder for the encrypted package"
@@ -232,18 +233,6 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
     def encrypt(self, dry_run=False):
         progress = GuiProgress()
         progress.updated.connect(self.progress_bar.setValue)
-        output_path = str(
-            self.app_data.encrypt_output_location
-            / "_".join(
-                filter(
-                    None,
-                    [
-                        datetime.now().astimezone().strftime(encrypt.DATE_FMT_FILENAME),
-                        self.app_data.encrypt_output_suffix,
-                    ],
-                )
-            )
-        )
         if not dry_run and self.app_data.config.sign_encrypted_data:
             pw = run_dialog(self, "Enter password for your GPG key")
             if pw is None:
@@ -260,7 +249,8 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
             verify_dtr=self.app_data.encrypt_verify_dtr,
             config=self.app_data.config,
             passphrase=pw,
-            output_name=output_path,
+            output=self.app_data.encrypt_output_location,
+            output_suffix=self.app_data.encrypt_output_suffix,
             dry_run=dry_run,
             offline=self.app_data.config.offline,
             compression_level=self.app_data.encrypt_compression_level,
