@@ -5,16 +5,15 @@ from typing import Iterator, Optional, Sequence, Callable
 from sett.core.error import UserError
 from .component import SelectionAction, ToolBar, LineEdit, Action, warning_callback
 from .parallel import Worker
-from .pyside import QtCore, QtGui, QtWidgets, QAction
+from .pyside import QtCore, QtGui, QtWidgets, QAction, open_window
 from ..core import crypt
+from ..core.crypt import gpg
 from ..utils.config import Config
 from ..workflows import (
     upload_keys as upload_keys_workflow,
     request_sigs as request_sigs_workflow,
 )
 
-Key = crypt.gpg.Key
-
 
 class KeysTab(QtWidgets.QWidget):
     def __init__(self, *args, **kwargs):
@@ -86,9 +85,11 @@ class KeysTab(QtWidgets.QWidget):
         layout.addWidget(self.text_panel, 3, 0, 1, 2)
         self.setLayout(layout)
 
-    def key_to_html(self, key: Key) -> str:
-        content = []
-        content.append("<table>")
+    def key_to_html(self, key: gpg.Key) -> str:
+        """Represent a PGP key as an HTML string"""
+
+        # Add key info (user ID, key ID, fingerprint, signatures).
+        content = ["<table>"]
         rows = [
             ("User ID", html.escape(str(key.uids[0]))),
             ("Key ID", key.key_id),
@@ -102,32 +103,52 @@ class KeysTab(QtWidgets.QWidget):
         content.append(
             "<br>".join(
                 [
-                    f"{html.escape(str(sig.issuer_uid))} {sig.issuer_key_id} {sig.signature_class}"
+                    f"{html.escape(str(sig.issuer_uid))} {sig.issuer_key_id} "
+                    f"{sig.signature_class}"
                     for sig in key.valid_signatures
                 ]
             )
         )
         content.append("</td></tr>")
 
+        # Add key validation info: is the key signed or not.
         content.append("</table>")
-        if key.key_type == crypt.gpg.KeyType.public:
+        if key.key_type == gpg.KeyType.public:
             try:
                 crypt.validate_pub_key(
-                    key,
-                    validation_keyid=self.app_data.config.key_validation_authority_keyid,
+                    key=key,
+                    signee_key=self.app_data.validation_authority_key,
                     keyserver_url=self.app_data.config.keyserver_url,
                 )
-                content.append('<p class="safe">This key has been verified</p>')
+                if self.app_data.validation_authority_key:
+                    content.append('<p class="safe">This key has been verified</p>')
+                elif self.app_data.config.key_authority_fingerprint:
+                    content.append(
+                        '<p class="info">This key could not be verified because the '
+                        "key validation authority's key ["
+                        + self.app_data.config.key_authority_fingerprint
+                        + "] is not available on your machine or is invalid.</p>"
+                    )
+                else:
+                    content.append(
+                        '<p class="info">This key could not be verified because no '
+                        "key validation authority is defined in the setting.</p>"
+                    )
             except UserError as e:
-                content.append(f'<p class="danger">{e}</p>')
+                # Note: changing "<email>" to "[email]" in the error message
+                # as the text between "< >" is not rendered.
+                content.append(
+                    f'<p class="danger">'
+                    f"{str(e).replace('<', '[').replace('>', ']')}</p>"
+                )
         else:
             content.append(
-                "<p>This is a private key. " "Private keys can not be signed.</p>"
+                "<p>This is a private key. Private keys cannot be signed.</p>"
             )
         return "".join(content)
 
     @staticmethod
-    def key_to_text(key: Key) -> str:
+    def key_to_text(key: gpg.Key) -> str:
         return f"User ID: {key.uids[0]}\n" f"Fingerprint: {key.fingerprint}"
 
     def create_public_keys_actions(self) -> Iterator[QAction]:
@@ -268,11 +289,11 @@ class KeysTab(QtWidgets.QWidget):
                 self._update_display(self.pub_keys_view.selectionModel().currentIndex())
                 msg.setIcon(QtWidgets.QMessageBox.Information)
                 msg.setText("Key has been imported.")
-                msg.exec_()
+                open_window(msg)
         except (UnicodeDecodeError, UserError) as e:
             msg.setIcon(QtWidgets.QMessageBox.Warning)
             msg.setText(format(e))
-            msg.exec_()
+            open_window(msg)
 
     def delete_keys(self):
         msg = QtWidgets.QMessageBox()
@@ -299,19 +320,21 @@ class KeysTab(QtWidgets.QWidget):
                     "this application. Please use an external software  "
                     "such as GnuPG (Linux, MacOS) or Kleopatra (Windows)."
                 )
-                msg_warn.exec_()
+                open_window(msg_warn)
                 continue
             msg.setDetailedText(self.key_to_text(key))
             if key is selected_keys[0]:
                 click_show_details(msgbox=msg)
-            status = msg.exec_()
+            status = open_window(msg)
             if status == QtWidgets.QMessageBox.Ok:
                 try:
-                    crypt.delete_keys([key.fingerprint], self.app_data.config.gpg_store)
+                    crypt.delete_pub_keys(
+                        [key.fingerprint], self.app_data.config.gpg_store
+                    )
                     self.pub_keys_view.selectionModel().clearSelection()
                 except UserError as e:
                     msg_warn.setText(format(e))
-                    msg_warn.exec_()
+                    open_window(msg_warn)
                 self.text_panel.clear()
         self.update_public_keys()
 
@@ -334,7 +357,7 @@ class KeysTab(QtWidgets.QWidget):
             msg.setDetailedText(self.key_to_text(key))
             if key is selected_keys[0]:
                 click_show_details(msgbox=msg)
-            status = msg.exec_()
+            status = open_window(msg)
             if status == QtWidgets.QMessageBox.Ok:
                 worker = Worker(
                     upload_keys_workflow.upload_keys,
@@ -411,11 +434,13 @@ class KeyDownloadDialog(QtWidgets.QDialog):
     def search_keys(self):
         self.btn_search.setEnabled(False)
         self.key_list_view.model().setStringList([])
+        show_warning = warning_callback("GPG key search error")
         worker = Worker(
             crypt.search_keyserver,
             self.search_string.text(),
             self.parent().app_data.config.keyserver_url,
             report_config=self.parent().app_data.config,
+            forward_errors=show_warning,
         )
         worker.signals.result.connect(
             lambda keys: self.key_list_view.model().setStringList(
@@ -431,12 +456,14 @@ class KeyDownloadDialog(QtWidgets.QDialog):
             key_ids.append(index.model().data(index).split()[-1])
         if key_ids:
             self.btn_download.setEnabled(False)
+            show_warning = warning_callback("GPG key search error")
             worker = Worker(
                 crypt.download_keys,
                 key_ids,
                 self.parent().app_data.config.keyserver_url,
                 self.parent().app_data.config.gpg_store,
                 report_config=self.parent().app_data.config,
+                forward_errors=show_warning,
             )
             worker.signals.result.connect(lambda x: self.parent().update_public_keys())
             worker.signals.finished.connect(lambda: self.btn_download.setEnabled(True))
@@ -500,7 +527,7 @@ class KeyGenDialog(QtWidgets.QDialog):
         self.text_pass.clear()
         self.text_pass_repeat.clear()
 
-    def post_key_creation(self, key: Key):
+    def post_key_creation(self, key: gpg.Key):
         msg = QtWidgets.QMessageBox(self)
         msg.setWindowTitle("GPG Key Generation")
         msg.setIcon(QtWidgets.QMessageBox.Information)
@@ -533,9 +560,9 @@ class KeyGenDialog(QtWidgets.QDialog):
                 f"gpg --gen-revoke {key.fingerprint}"
             )
         finally:
-            msg.exec_()
+            open_window(msg)
             if (
-                self.parent().app_data.config.key_validation_authority_keyid
+                self.parent().app_data.config.key_authority_fingerprint
                 and not self.parent().app_data.config.offline
             ):
                 request_signature(
@@ -572,7 +599,7 @@ def click_show_details(msgbox: QtWidgets.QMessageBox):
 
 
 def request_signature(
-    keys: Sequence[Key],
+    keys: Sequence[gpg.Key],
     config: Config,
     threadpool: QtCore.QThreadPool,
     parent: Optional[QtWidgets.QWidget] = None,
@@ -591,7 +618,7 @@ def request_signature(
         msg.setDetailedText(KeysTab.key_to_text(key))
         if key is keys[0]:
             click_show_details(msgbox=msg)
-        status = msg.exec_()
+        status = open_window(msg)
         if status == QtWidgets.QMessageBox.Yes:
             worker = Worker(
                 request_sigs_workflow.request_sigs,
@@ -610,4 +637,4 @@ def ok_message(title: str, msg: str, parent=None) -> Callable:
     msg_ok.setIcon(QtWidgets.QMessageBox.Information)
     msg_ok.setWindowTitle(title)
     msg_ok.setText(msg)
-    return msg_ok.exec_
+    return lambda: open_window(msg_ok)
