@@ -1,4 +1,5 @@
 import warnings
+import copy
 
 from sett import (
     APP_NAME_LONG,
@@ -15,7 +16,8 @@ from .encrypt_tab import EncryptTab
 from .keys_tab import KeysTab
 from .model import AppData
 from .parallel import Worker
-from .pyside import QtCore, QtGui, QtWidgets, QAction
+from .pyside import QtCore, QtGui, QtWidgets, QAction, open_window
+from .keys_refresh import load_authority_key_threaded
 from .settings_tab import SettingsTab, config_to_settings
 from .transfer_tab import TransferTab
 from ..core.versioncheck import check_version
@@ -29,12 +31,15 @@ class MainWindow(QtWidgets.QMainWindow):
         super().__init__()
         self.title = f"{APP_NAME_LONG} ({__version__})"
         self.app_data = AppData(config=self.get_config())
+        self.config_on_disk = copy.deepcopy(self.app_data.config)
+
         self.setWindowTitle(self.title)
         self.setMinimumSize(QtCore.QSize(900, 900))
         self.add_menu()
         self.add_tabs()
         self.add_status_bar()
         self.check_version()
+        load_authority_key_threaded(self.app_data)
 
     def get_config(self):
         with warnings.catch_warnings(record=True) as w:
@@ -104,13 +109,13 @@ class MainWindow(QtWidgets.QMainWindow):
 
     def closeEvent(self, event):
         if config_to_settings(self.app_data.config) != config_to_settings(
-            load_config()
+            self.config_on_disk
         ):
             reply = QtWidgets.QMessageBox.question(
                 self,
                 "Persist changed settings?",
-                "You made changes to settings you did not persist yet, "
-                "do you want to permanently persist them to your sett configuration file?",
+                "You made changes to settings you did not persist yet.\nDo you "
+                "want to permanently persist them to your sett configuration file?",
                 QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                 QtWidgets.QMessageBox.No,
             )
@@ -181,6 +186,6 @@ def open_url(url: str):
             msg_warn.setWindowTitle("Warning")
             msg_warn.setText(f"Unable to open URL at \n{url}.")
             msg_warn.setIcon(QtWidgets.QMessageBox.Warning)
-            msg_warn.exec_()
+            open_window(msg_warn)
 
     return open_url_template
