@@ -1,10 +1,12 @@
+import copy
 from typing import Optional, Dict
 
 from .component import PathInput, FieldMapping, create_slider, LineEdit, SpinBox
 from .pyside import QtCore, QtGui, QtWidgets
+from .keys_refresh import load_authority_key_threaded
 from ..utils import config
 from ..utils.config import Config
-from ..utils.validation import REGEX_URL
+from ..utils.validation import REGEX_URL, PACKAGE_SUFFIX
 
 
 LABEL_COMPRESSION_LEVEL = "Compression level"
@@ -23,8 +25,8 @@ def config_to_settings(cfg: Config) -> Dict:
 class SettingsTab(QtWidgets.QWidget):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.app_data = self.parent().app_data
-        self.config = self.app_data.config
+        self.parent_widget = self.parent()
+        self.config = self.parent_widget.app_data.config
 
         self.field_mapping = FieldMapping()
         self.layout: QtWidgets.QGridLayout = QtWidgets.QGridLayout()
@@ -33,26 +35,32 @@ class SettingsTab(QtWidgets.QWidget):
         self._add_header()
         self._add_settings()
         self._add_footer()
-        self._load_config()
+        self._populate_tab_with_config_values()
 
         self.setLayout(self.layout)
 
-    def _load_config(self):
-        """Overwrites contents of widgets with values in current config"""
+    def _populate_tab_with_config_values(self):
+        """Populate the fields of the tab with the app's config values"""
         settings_dict = config_to_settings(self.config)
         self.field_mapping.load(settings_dict)
 
     def _update_config(self, cfg: Config):
-        """Replaces the `config` with `cfg` and updates the contents of the widgets in the UI"""
-        self.app_data.config = cfg
+        """Replaces the `config` with `cfg` and updates the contents of the
+        widgets in the UI.
+        """
         self.config = cfg
-        self._load_config()
+        self._populate_tab_with_config_values()
+
+    def _save_config_to_disk(self):
+        """Write the current in-app config values to a config file on disk"""
+        config.save_config(config.config_to_dict(self.config))
+        self.parent_widget.config_on_disk = copy.deepcopy(self.config)
 
     def _add_header(self):
         top_label = QtWidgets.QLabel(
             "Changes you make are applied instantly during the current session.\n"
-            f"To persist changes across restarts, make sure to click the '{self.persist_btn_text}' "
-            f"button at the bottom."
+            f"To persist changes across restarts, make sure to click the "
+            f"'{self.persist_btn_text}' button at the bottom of the app."
         )
         top_label.setWordWrap(True)
         self.layout.addWidget(
@@ -124,21 +132,31 @@ class SettingsTab(QtWidgets.QWidget):
             "DTR verification and automatic PGP key downloading/updating is disabled.",
         )
 
+        self._add_widget_str(
+            encrypt_decrypt_group,
+            "Default package suffix",
+            "package_name_suffix",
+            "Default suffix for encrypted package name",
+            regex=PACKAGE_SUFFIX,
+        )
+
         pgp_keys_group = self._add_group("PGP keys")
 
         self._add_widget_str(
             pgp_keys_group,
-            "Key validation authority key ID",
-            "key_validation_authority_keyid",
-            "ID of the authority key. If a value is specified, "
-            "then only keys signed by this authority could be used.",
+            "Key validation authority fingerprint",
+            "key_authority_fingerprint",
+            "Fingerprint (40 characters) of the key validation authority's "
+            "PGP key. If a value is specified, only keys signed the "
+            "authority's key can be used.",
         )
 
         self._add_widget_path(
             pgp_keys_group,
-            "GPG config directory",
-            "gpg_config_dir",
-            "Path of the directory where GnuPG stores its configuration",
+            "GPG home directory",
+            "gpg_home_dir",
+            "Path of the directory where GnuPG stores its keyrings and "
+            "configuration files.",
             True,
         )
 
@@ -163,8 +181,8 @@ class SettingsTab(QtWidgets.QWidget):
             data_transfer_group,
             "DCC portal URL",
             "dcc_portal_url",
-            "URL of portal instance. "
-            "The portal is used for key signing and DTR (Data Transfer Request) validation.",
+            "URL of portal instance. The portal is used for key signing and "
+            "DTR (Data Transfer Request) validation.",
             REGEX_URL,
         )
 
@@ -230,12 +248,23 @@ class SettingsTab(QtWidgets.QWidget):
         persist_btn.setIcon(QtGui.QIcon(":icon/feather/save.png"))
         persist_btn.setStatusTip("Saves your current settings to the config file")
         persist_btn.clicked.connect(
-            lambda: config.save_config(config.config_to_dict(self.config))
+            # lambda: config.save_config(config.config_to_dict(self.config))
+            self._save_config_to_disk
         )
         self.layout.addWidget(persist_btn, self.layout.rowCount() + 1, 0, 1, 4)
 
     def _set_config(self, key: str, value):
-        return setattr(self.config, key, value)
+        """Updates the specified sett config attribute (key) with a new value"""
+        if value != getattr(self.config, key):
+            setattr(self.config, key, value)
+            if key == "key_authority_fingerprint":
+                # Only attempt to reload the key validation authority key if
+                # a proper fingerprint was entered. Otherwise we get a warning
+                # each time the user is entering a new character in the field.
+                if len(value) == 40:
+                    load_authority_key_threaded(self.parent_widget.app_data)
+                else:
+                    self.parent_widget.app_data.validation_authority_key = None
 
     def _get_config(self, key: str):
         return getattr(self.config, key)
