@@ -18,7 +18,7 @@ from .component import (
 )
 from .file_selection_widget import ArchiveOnlyFileSelectionWidget
 from .parallel import Worker
-from .pyside import QtCore, QtGui, QtWidgets, QAction
+from .pyside import QtCore, QtGui, QtWidgets, QAction, open_window
 from .. import protocols
 from ..core.error import UserError
 from ..utils.config import Connection, ConnectionStore
@@ -194,7 +194,7 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
                 QtWidgets.QMessageBox.Ok | QtWidgets.QMessageBox.Cancel
             )
             dialog.setDefaultButton(QtWidgets.QMessageBox.Cancel)
-            return dialog.exec_() == QtWidgets.QMessageBox.Ok
+            return open_window(dialog) == QtWidgets.QMessageBox.Ok
 
         def rename():
             label_old = connections_selection.currentText()
@@ -204,7 +204,7 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
                 forbidden=self.connections_model.stringList(),
                 default_input=label_old,
             )
-            if dialog.exec_():
+            if open_window(dialog):
                 label_new = dialog.text_field.text()
                 self.app_data.config.connections[
                     label_new
@@ -228,7 +228,7 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
                 label="New connection profile label:",
                 forbidden=self.connections_model.stringList(),
             )
-            if dialog.exec_():
+            if open_window(dialog):
                 label = dialog.text_field.text()
                 self.app_data.config.connections[label] = Connection(
                     protocol="sftp", parameters={}
