@@ -1,13 +1,15 @@
 import os
 import json
-from typing import Iterable, Tuple, Sequence
+from typing import Iterable, Tuple, Sequence, Optional
 from datetime import date
 from datetime import timedelta
+from dataclasses import dataclass
 import urllib.parse
 import urllib.request
 import base64
 
 from .multipart import encode as multipart_encode
+from . import protocol
 from ..utils.progress import (
     progress_iter,
     progress_iter_with_sub_progress,
@@ -15,49 +17,64 @@ from ..utils.progress import (
 )
 from ..core.error import UserError
 from ..core.request import build_opener
+from ..core.secret import Secret
 
 
-def required_password_args(**_) -> Sequence:
-    return ()
+@dataclass
+class Protocol(protocol.Protocol):
+    host: str
+    api_key: Secret
+    recipients: Sequence[str] = ()
+    subject: str = "BiomedIT file transfer"
+    message: str = ""
+    expires_at: Optional[date] = None
+    send_email: bool = False
+    progress: Optional[ProgressInterface] = None
+    chunk_size: int = 1024
 
+    def upload(
+        self,
+        files: Sequence[str],
+        progress: ProgressInterface = None,
+    ):
+        if self.expires_at is None:
+            expires_at = date.today() + timedelta(days=7)
 
-def upload(
-    files: Sequence[str],
-    recipients: Sequence[str],
-    host: str,
-    api_key: str,
-    subject: str = "BiomedIT file transfer",
-    message: str = "",
-    expires_at: date = None,
-    send_email: bool = False,
-    progress: ProgressInterface = None,
-    **kwargs,
-):
-    if expires_at is None:
-        expires_at = date.today() + timedelta(days=7)
+        attachment_ids = [
+            upload_attachment(
+                self.host,
+                self.api_key,
+                file_name,
+                progress=sub_progress,
+                chunk_size=self.chunk_size,
+            )
+            for file_name, sub_progress in progress_iter_with_sub_progress(
+                files, progress
+            )
+        ]
 
-    attachment_ids = [
-        upload_attachment(host, api_key, file_name, progress=sub_progress, **kwargs)
-        for file_name, sub_progress in progress_iter_with_sub_progress(files, progress)
-    ]
-
-    msg = {
-        "message": {
-            "recipients": recipients,
-            "subject": subject,
-            "message": message,
-            "expires_at": expires_at.strftime("%Y-%m-%d"),
-            "send_email": send_email,
-            "authorization": 3,
-            "attachments": attachment_ids,
+        msg = {
+            "message": {
+                "recipients": self.recipients,
+                "subject": self.subject,
+                "message": self.message,
+                "expires_at": expires_at.strftime("%Y-%m-%d"),
+                "send_email": self.send_email,
+                "authorization": 3,
+                "attachments": attachment_ids,
+            }
         }
-    }
-    return post(host + "message", json.dumps(msg).encode(), api_key, "application/json")
+        return post(
+            self.host + "message",
+            json.dumps(msg).encode(),
+            self.api_key,
+            "application/json",
+        )
 
 
 def upload_attachment(
     host: str,
-    api_key: str,
+    api_key: Secret,
     file_path: str,
     chunk_size: int = 1024,
     progress: ProgressInterface = None,
@@ -96,7 +113,7 @@ def upload_attachment(
     return attachment_id.decode()
 
 
-def chunk_iter(stream, chunk_size):
+def chunk_iter(stream, chunk_size: int):
     while True:
         chunk = stream.read(chunk_size)
         if not chunk:
@@ -104,7 +121,7 @@ def chunk_iter(stream, chunk_size):
         yield chunk
 
 
-def post(url, data, api_key, content_type):
+def post(url: str, data: bytes, api_key: Secret, content_type: str):
     auth = "Basic " + base64.b64encode((api_key + ":x").encode()).rstrip().decode()
     request = urllib.request.Request(
         url,
