@@ -8,10 +8,10 @@ from dataclasses import dataclass
 from pathlib import Path
 from typing import Tuple, Dict, Any, Optional
 
+import gpg_lite as gpg
 from libbiomedit.lib.deserialize import deserialize
 
 from .log import get_default_log_dir, create_logger
-from ..core.crypt import open_gpg_dir, get_default_gpg_config_dir
 from ..core.error import UserError
 from ..core.portal_api import PortalApi
 from .. import APP_NAME_SHORT
@@ -31,7 +31,7 @@ logger = create_logger(__name__)
 
 @dataclass(frozen=True)
 class Connection:
-    """dataclass holding config fo a connection (sftp / liquid files)"""
+    """dataclass holding config of a connection (sftp / liquid files)"""
 
     protocol: str
     parameters: Dict[str, Any]
@@ -46,8 +46,10 @@ class Config:
 
     dcc_portal_url: str = "https://portal.dcc.sib.swiss"
     keyserver_url: Optional[str] = "keyserver.dcc.sib.swiss"
-    gpg_config_dir: str = get_default_gpg_config_dir()
-    key_validation_authority_keyid: Optional[str] = "881685B5EE0FCBD3"
+    gpg_home_dir: str = gpg.get_default_gnupg_home_dir()
+    key_authority_fingerprint: Optional[
+        str
+    ] = "B37CE2A101EBFA70941DF885881685B5EE0FCBD3"
     sign_encrypted_data: bool = True
     always_trust_recipient_key: bool = True
     repo_url: str = "https://pypi.org"
@@ -63,6 +65,7 @@ class Config:
     default_sender: Optional[str] = None
     gui_quit_confirmation: bool = True
     compression_level: int = 5
+    package_name_suffix: Optional[str] = None
 
     def __post_init__(self):
         for url in ("dcc_portal_url", "repo_url"):
@@ -74,7 +77,7 @@ class Config:
 
     @property
     def gpg_store(self):
-        return open_gpg_dir(self.gpg_config_dir)
+        return open_gpg_dir(self.gpg_home_dir)
 
 
 class ConnectionStore:
@@ -134,18 +137,23 @@ class ConnectionStore:
 
 
 def load_config() -> Config:
-    """Loads the config, returning a Config object"""
+    """Loads the config, returning a Config object."""
+
+    # TODO: remove migration (see issue 283)
+    migrate_from_keyid_to_fingerprint()
+
     cfg_dct = {}
     try:
         cfg_dct = sys_config_dict()
         cfg_dct.update(load_config_dict(get_config_file()))
     except UserError as e:
         warnings.warn(format(e))
+
     return deserialize(Config)(cfg_dct)
 
 
 def save_config(config: Dict, path: Optional[str] = None) -> str:
-    """Save config to a file
+    """Save config to a file.
     :return: path of the created config file.
     """
     config_file = Path(get_config_file() if path is None else path)
@@ -165,19 +173,19 @@ def create_config() -> str:
 
 
 def config_to_dict(config: Config) -> dict:
-    """Converts a Config object into a dict"""
+    """Converts a Config object into a dict."""
     return dataclasses.asdict(config)
 
 
 def default_config() -> Config:
-    """Creates a new Config object with default values"""
+    """Creates a new Config object with default values."""
     return deserialize(Config)(sys_config_dict())
 
 
 def sys_config_dict() -> dict:
-    """On linux only: try to load global sys config.
-    If the env variable `SYSCONFIG` is set search there, else in
-    `/etc/{APP_NAME_SHORT}`"""
+    """On linux only: try to load global sys config. If the env variable
+    SYSCONFIG is set search there, else in /etc/{APP_NAME_SHORT}.
+    """
     if platform.system() == "Linux":
         sys_cfg_dir = os.environ.get("SYSCONFIG", os.path.join("/etc", APP_NAME_SHORT))
         return load_config_dict(os.path.join(sys_cfg_dir, CONFIG_FILE_NAME))
@@ -185,7 +193,7 @@ def sys_config_dict() -> dict:
 
 
 def load_config_dict(path: str) -> dict:
-    """Load raw config as a dict"""
+    """Load raw config as a dict."""
     try:
         with open(path) as f:
             return json.load(f)
@@ -223,3 +231,56 @@ def get_config_dir() -> str:
     return os.path.join(
         os.path.expanduser("~"), *conf_sub_dir_by_os[platform.system()], APP_NAME_SHORT
     )
+
+
+def open_gpg_dir(gpg_dir: str) -> gpg.GPGStore:
+    """Open the database inside a GnuPG directory and return it as a gpg
+    object.
+
+    :param gpg_dir: path of the GnuPG directory to open.
+    :return: a gpg-lite GPGStore
+    :raises UserError:
+    """
+    if not Path(gpg_dir).is_dir():
+        os.makedirs(gpg_dir, mode=0o700)
+
+    try:
+        return gpg.GPGStore(gnupg_home_dir=gpg_dir)
+    except ValueError:
+        raise UserError(f"unable to open GnuPG directory [{gpg_dir}].") from None
+
+
+def migrate_from_keyid_to_fingerprint():
+    """Temporary migration to auto-update user's config files after changes
+    from https://gitlab.com/biomedit/sett/-/merge_requests/56
+    """
+    config_file = get_config_file()
+    config_needs_update = False
+
+    try:
+        config_dict = load_config_dict(config_file)
+    except UserError:
+        return
+
+    if "gpg_config_dir" in config_dict:
+        config_dict["gpg_home_dir"] = config_dict.pop("gpg_config_dir")
+        config_needs_update = True
+
+    if "key_validation_authority_keyid" in config_dict:
+        fingerprint = Config().key_authority_fingerprint
+        key_id = config_dict.pop("key_validation_authority_keyid")
+        if key_id == fingerprint[-16:]:
+            config_dict["key_authority_fingerprint"] = fingerprint
+        else:
+            config_dict["key_authority_fingerprint"] = key_id
+            warnings.warn(
+                f"This new version of {APP_NAME_SHORT} requires the full "
+                "fingerprint of the key validation authority. Please update "
+                "the 'key_authority_fingerprint' field in your config file: "
+                f"{config_file}"
+            )
+
+        config_needs_update = True
+
+    if config_needs_update:
+        save_config(config_dict, path=config_file)
