@@ -75,11 +75,13 @@ def cleanup_error_reports(log_dir: str):
     delete_files(*old_crash_reports)
 
 
-def log_exceptions(logger: Logger) -> Callable[[Callable], Callable]:
+def log_exceptions(
+    logger: Logger, reraise_warnings: bool = False
+) -> Callable[[Callable], Callable]:
     """A function decorator catching the first exception of instance contained
     in or equal to `EXCEPTION_TYPES`, occurring while executing the decorated
-    function, logging an error and reraises the exception for further error handling.
-    Also turns warnings into log entries
+    function, logging an error and reraises the exception for further error
+    handling. It also turns warnings into log entries.
     """
 
     def decorator(f: Callable) -> Callable:
@@ -90,8 +92,15 @@ def log_exceptions(logger: Logger) -> Callable[[Callable], Callable]:
                     warnings.simplefilter("always")
                     r = f(*args, **kwargs)
                     for warning in w:
-                        logger.warning(format(warning.message))
-                    return r
+                        # Note: the reason that "\n" are removed is so that
+                        # each warning gets logged on a single line.
+                        logger.warning(format(warning.message).replace("\n", " "))
+
+                if reraise_warnings:
+                    for warning in w:
+                        warnings.warn(format(warning.message))
+                return r
+
             except EXCEPTION_TYPES as e:
                 logger.error(format(e))
                 raise
