@@ -1,18 +1,70 @@
+import enum
 import io
-import platform
-from functools import wraps
 import logging
 import logging.handlers
-from typing import Callable, Tuple, Dict, cast, ContextManager, Optional
-from pathlib import Path
+import platform
 import time
 from contextlib import contextmanager
+from functools import wraps
+from pathlib import Path
+from typing import Callable, Tuple, Dict, cast, ContextManager, Optional
+
+import colorama  # type: ignore
 
 from .. import RUNTIME_INFO
 
 LOG_FORMAT = "[%(levelname)s] %(message)s"
+LOG_FORMAT_WITH_EMOJI = "%(levelname)s  %(message)s"
 LOG_FORMAT_WITH_TIME = "[%(asctime)s] " + LOG_FORMAT
 
+colorama.init()
+
+
+class LogStyle(enum.Enum):
+    EMOJI = enum.auto()
+    TERMINAL = enum.auto()
+    HTML = enum.auto()
+
+    def get_style(self, log_level: int) -> Optional[str]:
+        style = {
+            LogStyle.EMOJI: {
+                logging.DEBUG: "🔬",
+                logging.INFO: "ℹ️",
+                logging.WARNING: "⚠️",
+                logging.ERROR: "💥",
+                logging.CRITICAL: "☢️",
+            },
+            LogStyle.TERMINAL: {
+                logging.WARNING: colorama.Fore.YELLOW,
+                logging.ERROR: colorama.Fore.RED,
+                logging.CRITICAL: colorama.Fore.RED,
+            },
+            LogStyle.HTML: {
+                logging.DEBUG: "gray",
+                logging.INFO: "black",
+                logging.WARNING: "orange",
+                logging.ERROR: "red",
+                logging.CRITICAL: "red",
+            },
+        }
+        return style[self].get(log_level)
+
+
+class EmojiLevelFormatter(logging.Formatter):
+    def format(self, record: logging.LogRecord):
+        emoji = LogStyle.EMOJI.get_style(record.levelno)
+        if emoji:
+            record.levelname = emoji
+        return super().format(record)
+
+
+class ColoredTerminalFormatter(logging.Formatter):
+    def format(self, record: logging.LogRecord) -> str:
+        color = LogStyle.TERMINAL.get_style(record.levelno)
+        if color:
+            return f"{color}{super().format(record)}{colorama.Fore.RESET}"
+        return super().format(record)
+
 
 @contextmanager
 def log_task(
@@ -187,7 +239,11 @@ def log_to_stream(
     if logger is None:
         logger = logging.getLogger()
     console_handler = logging.StreamHandler()
-    console_handler.setFormatter(logging.Formatter(LOG_FORMAT))
+    console_handler.setFormatter(
+        ColoredTerminalFormatter(LOG_FORMAT)
+        if platform.system() == "Windows"
+        else EmojiLevelFormatter(LOG_FORMAT_WITH_EMOJI)
+    )
     console_handler.setLevel(logger_level)
     logger.addHandler(console_handler)
 
