@@ -6,7 +6,7 @@ from typing import List, Optional
 
 from ..core.error import UserError
 from ..core.archive import (
-    check_tar,
+    check_package,
     extract,
     unpack_from_stream,
     METADATA_FILE,
@@ -17,10 +17,10 @@ from ..core.archive import (
 )
 from ..core.crypt import (
     decrypt as core_decrypt,
-    extract_pub_key_ids,
-    validated_keys_by_ids,
+    retrieve_refresh_and_validate_keys,
     fingerprint2keyid,
     verify_metadata_signature,
+    gpg,
 )
 from ..core.filesystem import DeleteDirOnError, unique_filename
 from ..core.checksum import (
@@ -63,20 +63,20 @@ def decrypt(
     )
 
     with logger.log_task("Input data check"):
-        for tar_name in files:
-            check_tar(tar_name)
+        for archive_path in files:
+            check_package(archive_path)
 
     if dry_run:
         logger.info("Dry run completed successfully")
         return
 
-    for tar_file in files:
+    for archive_file in files:
         # Reset progress for each tar file
         if progress is not None:
             progress.update(0.0)
-        logger.info("Untar file %s", tar_file)
-        decrypt_tar(
-            tar_file,
+        logger.info("Processing file %s", archive_file)
+        decrypt_archive(
+            archive_file,
             passphrase=passphrase,
             output_dir=output_dir,
             config=config,
@@ -86,8 +86,8 @@ def decrypt(
         logger.info("Data decryption completed successfully.")
 
 
-def decrypt_tar(
-    tar_file: str,
+def decrypt_archive(
+    archive_file: str,
     passphrase: Optional[Secret],
     output_dir: str,
     config: Config,
@@ -95,29 +95,31 @@ def decrypt_tar(
     progress: Optional[ProgressInterface] = None,
 ):
 
-    # To avoid overwriting files, each tarball is untarred in a directory
-    # that has the same name as the tar file minus the .tar extension.
+    # To avoid overwriting files, each archive is unpacked in a directory
+    # that has the same name as the archive file minus its extension.
     out_dir = unique_filename(
-        os.path.splitext(os.path.join(output_dir, os.path.basename(tar_file)))[0]
+        os.path.splitext(os.path.join(output_dir, os.path.basename(archive_file)))[0]
     )
 
     with DeleteDirOnError(out_dir):
         with logger.log_task("Verifying signatures..."):
             verify_metadata_signature(
-                tar_file,
-                config.gpg_store,
-                config.key_validation_authority_keyid,
-                config.keyserver_url,
+                tar_file=archive_file,
+                gpg_store=config.gpg_store,
+                signee_fingerprint=None,
+                key_authority_fingerprint=config.key_authority_fingerprint,
+                keyserver_url=config.keyserver_url,
+                validate_key_origin=False,
             )
 
-        with logger.log_task("Verifying encryption keys..."), open(
-            tar_file, "rb"
-        ) as tar_obj, extract(tar_obj, DATA_FILE_ENCRYPTED) as f_data:
-            keys = validated_keys_by_ids(
-                extract_pub_key_ids(f_data),
-                config.gpg_store,
-                config.key_validation_authority_keyid,
-                config.keyserver_url,
+        with logger.log_task("Verifying encryption keys..."), extract(
+            archive_file, DATA_FILE_ENCRYPTED
+        ) as f_data:
+            keys = retrieve_refresh_and_validate_keys(
+                key_search_terms=gpg.extract_key_id(f_data),
+                gpg_store=config.gpg_store,
+                key_authority_fingerprint=config.key_authority_fingerprint,
+                keyserver_url=config.keyserver_url,
             )
             logger.info(
                 "Data encrypted for:\n%s",
@@ -127,12 +129,10 @@ def decrypt_tar(
                 ),
             )
 
-        with logger.log_task("Verifying checksums..."), open(
-            tar_file, "rb"
-        ) as tar_obj, subprogress(
+        with logger.log_task("Verifying checksums..."), subprogress(
             progress, step_completion_increase=0.05
         ) as scaled_progress, extract_with_progress(
-            tar_obj, scaled_progress, METADATA_FILE, DATA_FILE_ENCRYPTED
+            archive_file, scaled_progress, METADATA_FILE, DATA_FILE_ENCRYPTED
         ) as (
             f_metadata,
             f_data,
@@ -141,12 +141,10 @@ def decrypt_tar(
             if metadata.checksum.lower() != compute_checksum_sha256(f_data):
                 raise UserError(f"Checksum mismatch for {f_data.name}")
 
-        with logger.log_task("Decrypting data..."), open(
-            tar_file, "rb"
-        ) as tar_obj, subprogress(
+        with logger.log_task("Decrypting data..."), subprogress(
             progress, step_completion_increase=0.9
         ) as scaled_progress, extract_with_progress(
-            tar_obj, scaled_progress, DATA_FILE_ENCRYPTED
+            archive_file, scaled_progress, DATA_FILE_ENCRYPTED
         ) as f_data:
             os.makedirs(out_dir, exist_ok=True)
             if decrypt_only:
@@ -165,11 +163,11 @@ def decrypt_tar(
                     passphrase=passphrase,
                 )
                 log_files(unpacked)
-            sender_sig_keys = validated_keys_by_ids(
-                map(fingerprint2keyid, sender_fprs),
-                config.gpg_store,
-                config.key_validation_authority_keyid,
-                config.keyserver_url,
+            sender_sig_keys = retrieve_refresh_and_validate_keys(
+                key_search_terms=map(fingerprint2keyid, sender_fprs),
+                gpg_store=config.gpg_store,
+                key_authority_fingerprint=config.key_authority_fingerprint,
+                keyserver_url=config.keyserver_url,
             )
             logger.info(
                 "Data signed by:\n%s",
