@@ -1,9 +1,11 @@
+from datetime import datetime
 import io
 import json
 import os
 from functools import partial
 from pathlib import Path
-from typing import List, Optional, Callable, Tuple, Union
+from typing import List, Optional, Callable, Tuple, Union, Iterable
+from zipfile import ZipFile, ZipInfo, ZIP_STORED
 
 from ..utils.config import Config
 from ..utils.log import create_logger, log_timing
@@ -19,20 +21,23 @@ from ..core.archive import (
     CONTENT_FOLDER,
 )
 from ..core.filesystem import (
+    delete_file_on_error,
     search_files_recursively,
     check_file_read_permission,
-    delete_files,
 )
 from ..core.crypt import (
-    retrieve_keys,
+    retrieve_refresh_and_validate_keys,
     encrypt_and_sign,
     detach_sign_file,
     check_password,
+    search_priv_key,
+    gpg,
 )
 from ..core.checksum import write_checksums, compute_checksum_on_write
 from ..core.error import UserError
 from ..core.metadata import MetaData, alnum_str, Purpose, HexStr1024, HexStr256
 from ..core.secret import Secret
+from ..core.portal_api import PortalApi
 from ..utils.progress import ProgressInterface, subprogress, progress_file_iter
 
 DATE_FMT_FILENAME = "%Y%m%dT%H%M%S"
@@ -106,7 +111,6 @@ def check_integer_in_range(min_value: int, max_value: int) -> Callable:
 check_compression_level = check_integer_in_range(min_value=0, max_value=9)
 
 
-# pylint: disable=too-many-statements
 @log_timing(logger)
 def encrypt(
     files: List[str],
@@ -116,7 +120,8 @@ def encrypt(
     dtr_id: Optional[int] = None,
     sender: Optional[str] = None,
     passphrase: Optional[Secret] = None,
-    output_name: Optional[str] = None,
+    output: Optional[str] = None,
+    output_suffix: Optional[str] = None,
     dry_run: bool = False,
     offline: bool = False,
     verify_dtr: bool = True,
@@ -155,13 +160,17 @@ def encrypt(
         )
         if dtr_id is None and verify_dtr:
             raise UserError("DTR (Data Transfer Request) ID is missing.")
-        if output_name:
-            check_arg_value(Path(output_name).name, "output name", output_name_str)
+        if output:
+            check_arg_value(Path(output).name, "output name", output_name_str)
             check_arg_value(
-                Path(output_name).parent.as_posix(),
+                Path(output).as_posix()
+                if Path(output).is_dir()
+                else Path(output).parent.as_posix(),
                 "path in output name",
                 output_path_str,
             )
+        if output_suffix:
+            check_arg_value(output_suffix, "output suffix", output_name_str)
 
     with logger.log_task("Retrieve sender and recipient GnuPG keys"):
         # Retrieve the sender's public and private keys, as well as the
@@ -174,14 +183,19 @@ def encrypt(
         #
         # The sender/recipient information can be either an email, a keyID or
         # a full fingerprint.
-        sender_pub_key, recipients_pub_key = retrieve_keys(
+        sender_pub_key, *recipients_pub_key = retrieve_refresh_and_validate_keys(
+            key_search_terms=(sender, *recipient),
             gpg_store=config.gpg_store,
-            sender=sender,
-            recipients=recipient,
-            validation_keyid=config.key_validation_authority_keyid,
-            keyserver_url=config.keyserver_url,
-            with_refresh_keys=not offline,
+            key_authority_fingerprint=config.key_authority_fingerprint,
+            keyserver_url=config.keyserver_url if not offline else None,
+            validate_key_origin=False,
         )
+
+        # Verify a private key matching the user's public key exists. The key
+        # itself is not needed because it shares the fingerprint with the
+        # public key.
+        search_priv_key(sender_pub_key.fingerprint, config.gpg_store)
+
         logger.info(
             "found the following keys:\n%s\n%s",
             f"sender key: {sender_pub_key.uids[0]}, "
@@ -194,29 +208,32 @@ def encrypt(
             ),
         )
 
-        if dtr_id is not None:
-            if purpose is None:
-                raise UserError("DTR ID specified but `purpose` is missing")
-            if not offline and verify_dtr:
-                try:
-                    project_id = config.portal_api.verify_transfer(
-                        metadata=MetaData(
-                            transfer_id=dtr_id,
-                            sender=HexStr1024(sender_pub_key.fingerprint),
-                            recipients=[
-                                HexStr1024(key.fingerprint)
-                                for key in recipients_pub_key
-                            ],
-                            checksum=HexStr256("0" * 64),
-                            purpose=purpose,
-                        ),
-                        filename="missing",
-                    )
-                    logger.info(
-                        "DTR ID '%s' is valid for project '%s'", dtr_id, project_id
-                    )
-                except RuntimeError as e:
-                    raise UserError(format(e)) from e
+        project_code = determine_project_code(
+            dtr_id,
+            purpose,
+            config.portal_api if not offline and verify_dtr else None,
+            sender_pub_key,
+            recipients_pub_key,
+        )
+
+        timestamp = datetime.now().astimezone()
+        # The default value for the output name is based on date and time
+        # when the script is being run.
+        # Example output name is "20191011T145012".
+        output_name = generate_output_archive_name(
+            output,
+            default="_".join(
+                filter(
+                    None,
+                    [
+                        project_code,
+                        timestamp.strftime(DATE_FMT_FILENAME),
+                        output_suffix or config.package_name_suffix,
+                    ],
+                )
+            ),
+        )
+
     if dry_run:
         logger.info("dry run completed successfully.")
         return None
@@ -246,54 +263,59 @@ def encrypt(
                 )
             )
 
-    with logger.log_task("Compress and encrypt input data " "[this can take a while]"):
+    with logger.log_task("Compress and encrypt input data [this can take a while]"):
         # Encryption is done with the recipient's public key and the optional
         # signing with the user's (i.e sender) private key. The user's private
         # key passphrase is needed to sign the encrypted file.
-        encrypted_file = os.path.join(config.temp_dir, DATA_FILE_ENCRYPTED)
         encrypted_checksum_buf = io.StringIO()
-        with subprogress(progress, step_completion_increase=0.75) as scaled_progress:
-            # Create a tar archive containing all input files
-            archive_content: Tuple[ArchiveFileBase, ...] = (
-                ArchiveInMemoryFile(CHECKSUM_FILE, checksums),
-            ) + tuple(
-                ArchiveFile(a_path, f)
-                for a_path, f in zip(
-                    archive_paths,
-                    progress_file_iter(
-                        files=files_to_encrypt, mode="rb", progress=scaled_progress
-                    ),
+        with delete_file_on_error(output_name), ZipFile(
+            output_name, mode="w", compression=ZIP_STORED
+        ) as zip_obj:
+            with subprogress(progress, step_completion_increase=0.8) as scaled_progress:
+                # Create a tar archive containing all input files
+                archive_content: Tuple[ArchiveFileBase, ...] = (
+                    ArchiveInMemoryFile(CHECKSUM_FILE, checksums),
+                ) + tuple(
+                    ArchiveFile(a_path, f)
+                    for a_path, f in zip(
+                        archive_paths,
+                        progress_file_iter(
+                            files=files_to_encrypt, mode="rb", progress=scaled_progress
+                        ),
+                    )
                 )
-            )
-            with open(encrypted_file, "wb") as fout:
-                encrypt_and_sign(
-                    source=partial(
-                        write_tar,
-                        archive_content,
-                        compress_level=compression_level,
-                        compress_algo="gz",
-                    ),
-                    output=partial(
-                        compute_checksum_on_write,
-                        fout=fout,
-                        checksum_buffer=encrypted_checksum_buf,
-                    ),
-                    gpg_store=config.gpg_store,
-                    recipients_fingerprint=[
-                        key.fingerprint for key in recipients_pub_key
-                    ],
-                    signature_fingerprint=sender_pub_key.fingerprint
-                    if config.sign_encrypted_data
-                    else None,
-                    passphrase=enforce_passphrase(passphrase)
-                    if config.sign_encrypted_data
-                    else None,
-                    always_trust=config.always_trust_recipient_key,
-                )
-        encrypted_checksum = encrypted_checksum_buf.read()
+                with zip_obj.open(
+                    ZipInfo(DATA_FILE_ENCRYPTED, date_time=timestamp.timetuple()[:6]),
+                    mode="w",
+                    force_zip64=True,
+                ) as fout:
+                    encrypt_and_sign(
+                        source=partial(
+                            write_tar,
+                            archive_content,
+                            compress_level=compression_level,
+                            compress_algo="gz",
+                        ),
+                        output=partial(
+                            compute_checksum_on_write,
+                            fout=fout,
+                            checksum_buffer=encrypted_checksum_buf,
+                        ),
+                        gpg_store=config.gpg_store,
+                        recipients_fingerprint=[
+                            key.fingerprint for key in recipients_pub_key
+                        ],
+                        signature_fingerprint=sender_pub_key.fingerprint
+                        if config.sign_encrypted_data
+                        else None,
+                        passphrase=enforce_passphrase(passphrase)
+                        if config.sign_encrypted_data
+                        else None,
+                        always_trust=config.always_trust_recipient_key,
+                    )
+                encrypted_checksum = encrypted_checksum_buf.read()
 
-    try:
-        with logger.log_task("Generate metadata file"):
+            logger.info("Generating metadata")
             # Create a dictionary with all the info we want to store in the
             # .json file, then pass this dictionary to json.dump that will
             # convert it to a json file.
@@ -306,54 +328,82 @@ def encrypt(
                 checksum=HexStr256(encrypted_checksum),
                 compression_algorithm="gzip" if compression_level > 0 else "",
             )
-            metadata_bytes = json.dumps(MetaData.asdict(metadata), indent=4).encode()
-            metadata_signature_bytes = b""
-            if config.sign_encrypted_data:
-                metadata_signature_bytes = detach_sign_file(
-                    metadata_bytes,
-                    sender_pub_key.fingerprint,
-                    enforce_passphrase(passphrase),
-                    config.gpg_store,
-                )
-
-        with logger.log_task("Generate final tarball"):
-            # The default value for the output name is based on date and time
-            # when the script is being run.
-            # Example output name is "20191011T145012".
-            output_name = form_output_tar_name(
-                output_name, default=metadata.timestamp.strftime(DATE_FMT_FILENAME)
+            metadata_bytes, metadata_signature_bytes = byte_encode_metadata(
+                metadata,
+                config.gpg_store,
+                passphrase,
+                sender_pub_key if config.sign_encrypted_data else None,
             )
+
             in_memory_files = (
                 (METADATA_FILE, metadata_bytes),
                 (METADATA_FILE_SIG, metadata_signature_bytes),
             )
-            with subprogress(
-                progress, step_completion_increase=0.05
-            ) as scaled_progress:
-                archive_content = tuple(
-                    ArchiveInMemoryFile(path, content)
-                    for path, content in in_memory_files
-                ) + tuple(
-                    ArchiveFile(os.path.relpath(f.name, start=config.temp_dir), f)
-                    for f in progress_file_iter(
-                        files=[encrypted_file], mode="rb", progress=scaled_progress
-                    )
+            for name, contents in in_memory_files:
+                zip_obj.writestr(
+                    ZipInfo(name, date_time=datetime.utcnow().timetuple()[:6]),
+                    contents,
                 )
-                write_tar(content=archive_content, output=output_name, compress_level=0)
-    finally:
-        delete_files(encrypted_file)
 
     logger.info("Completed data encryption: %s", output_name)
     return output_name
 
 
+def byte_encode_metadata(
+    metadata: MetaData,
+    gpg_store: gpg.GPGStore,
+    passphrase: Optional[Secret],
+    sender_pub_key: Optional[gpg.Key],
+) -> Tuple[bytes, bytes]:
+    metadata_bytes = json.dumps(MetaData.asdict(metadata), indent=4).encode()
+    metadata_signature_bytes = b""
+    if sender_pub_key is not None:
+        metadata_signature_bytes = detach_sign_file(
+            metadata_bytes,
+            sender_pub_key.fingerprint,
+            enforce_passphrase(passphrase),
+            gpg_store,
+        )
+    return metadata_bytes, metadata_signature_bytes
+
+
+def determine_project_code(
+    dtr_id: Optional[int],
+    purpose: Optional[Purpose],
+    portal_api: Optional[PortalApi],
+    sender_pub_key: gpg.Key,
+    recipients_pub_key: Iterable[gpg.Key],
+) -> Optional[str]:
+    if dtr_id is None:
+        return None
+    if purpose is None:
+        raise UserError("DTR ID specified but `purpose` is missing")
+    if portal_api is None:
+        return None
+    try:
+        project_code = portal_api.verify_transfer(
+            metadata=MetaData(
+                transfer_id=dtr_id,
+                sender=HexStr1024(sender_pub_key.fingerprint),
+                recipients=[HexStr1024(key.fingerprint) for key in recipients_pub_key],
+                checksum=HexStr256("0" * 64),
+                purpose=purpose,
+            ),
+            filename="missing",
+        )
+        logger.info("DTR ID '%s' is valid for project '%s'", dtr_id, project_code)
+    except RuntimeError as e:
+        raise UserError(format(e)) from e
+    return project_code
+
+
 def enforce_passphrase(passphrase: Optional[Secret]) -> Secret:
     if passphrase is None:
         raise ValueError("No password given")
     return passphrase
 
 
-def form_output_tar_name(output_name: Optional[str], default: str) -> str:
+def generate_output_archive_name(output_name: Optional[str], default: str) -> str:
     """Define the path + name of the output tarball file of the encrypt
     workflow. If output_name does not contain any path information, the
     output directory is set to the current working directory.
@@ -366,9 +416,9 @@ def form_output_tar_name(output_name: Optional[str], default: str) -> str:
         output_name = default
     if Path(output_name).is_dir():
         output_name = os.path.join(output_name, default)
-    # Add '.tar' extension to output name if needed.
-    if not output_name.endswith(".tar"):
-        output_name = output_name + ".tar"
+    # Add '.zip' extension to output name if needed.
+    if not output_name.endswith(".zip"):
+        output_name = output_name + ".zip"
 
     # If output_name does not contain any path info, the output path is
     # set to the current working directory.
