@@ -4,13 +4,8 @@ import json
 from ..utils.config import Config
 from ..utils.log import create_logger, log_timing
 from ..utils.progress import ProgressInterface
-from ..core.crypt import (
-    get_recipient_email,
-    extract_pub_key_ids,
-    validated_keys_by_ids,
-    gpg,
-)
-from ..core.archive import check_tar, extract, METADATA_FILE, DATA_FILE_ENCRYPTED
+from ..core.crypt import get_recipient_email, retrieve_refresh_and_validate_keys, gpg
+from ..core.archive import check_package, extract, METADATA_FILE, DATA_FILE_ENCRYPTED
 from ..core.metadata import load_metadata
 from ..core.error import UserError
 from ..protocols import Protocol
@@ -39,25 +34,25 @@ def transfer(
         str(dry_run),
     )
 
-    with logger.log_task("Input files verification"):
-        for tar in files:
-            check_tar(tar)
+    with logger.log_task("Input data check"):
+        for archive_path in files:
+            check_package(archive_path)
 
     files_by_recipient: Dict[Tuple[gpg.Key, ...], List[str]] = {}
     with logger.log_task("Extracting destination for each file to transfer"):
-        for tar in files:
-            logger.info("\t-> %s", tar)
-            with extract(open(tar, "rb"), METADATA_FILE, DATA_FILE_ENCRYPTED) as (
+        for archive_path in files:
+            logger.info("\t-> %s", archive_path)
+            with extract(archive_path, METADATA_FILE, DATA_FILE_ENCRYPTED) as (
                 metadata,
                 encrypted_file,
             ):
                 metadata = json.load(metadata)
                 keys = tuple(
-                    validated_keys_by_ids(
-                        extract_pub_key_ids(encrypted_file),
-                        config.gpg_store,
-                        config.key_validation_authority_keyid,
-                        config.keyserver_url,
+                    retrieve_refresh_and_validate_keys(
+                        key_search_terms=gpg.extract_key_id(encrypted_file),
+                        gpg_store=config.gpg_store,
+                        key_authority_fingerprint=config.key_authority_fingerprint,
+                        keyserver_url=config.keyserver_url,
                     )
                 )
             metadata = load_metadata(metadata)
@@ -66,20 +61,20 @@ def transfer(
             if verify_dtr:
                 if metadata.transfer_id is None:
                     raise UserError(
-                        "DTR (Data Transfer Request) ID is " "missing in file metadata."
+                        "DTR (Data Transfer Request) ID is missing in file metadata."
                     )
                 try:
-                    project_id = config.portal_api.verify_transfer(
-                        metadata=metadata, filename=tar
+                    project_code = config.portal_api.verify_transfer(
+                        metadata=metadata, filename=archive_path
                     )
                     logger.info(
                         "DTR ID '%s' is valid for project '%s'",
                         metadata.transfer_id,
-                        project_id,
+                        project_code,
                     )
                 except RuntimeError as e:
                     raise UserError(format(e)) from e
-            files_by_recipient.setdefault(keys, []).append(tar)
+            files_by_recipient.setdefault(keys, []).append(archive_path)
 
     if dry_run:
         logger.info("Dry run completed successfully")
